/*******************************************************************************************************
********************************************************************************************************
********* This program is designed for readers interested in reproducing the results in the    *********
********* the paper Li, Lin, and Zhang [2017]. Please note that the file labelled "value_list" *********
********* needs to first be cleaned by the authors based on the segment reporting data (WRDS). *********
*********                                                                                      *********
********* CODE1 is used to clean the raw data from WRDS and construct the dependent variables. *********
********* In addition, it is also used to construct the controls used in Ellis et al. [2012].  *********
********* The list of input datasets includes:                                                 *********
********* (1) segment: the segment reporting data; (2) Value_list: the hand cleaned data       *********
********* on the name of clients; (3) Idd: the IDD status of each state (Klasa et al.[2017]    *********
********* (4) real_loca : the 10-K Header data; (5) north: compustat (accounting data);(6)     *********
********* mkv : CRSP (stock return data)                                                       *********
********************************************************************************************************
********************************************************************************************************/
libname idd "C:\Users\Dropbox\IDD and disclosure\raw data";

****Only keep the obs with customer type of COMPANY*****;
data seg;
set idd.segment;
if CTYPE^="COMPANY" then delete; run;


**Import the list of reported and unreported identity (hand cleaned data) (list.xls)********;

data value_list;
set idd.value_list;
run;

proc sort data=seg; by CNMS;run;
proc sort data=value_list; by CNMS;run;

data seg;
merge seg value_list; by CNMS;run;

data seg;
set seg;
if unreported=. then unreported=0;
if CNMS="" then unreported=1;
reported=1-unreported;
sale_r=SALECS*reported;
CIK2=CIK*1;
run;

****If firm i do not report any of its customers in year t, then define it as a non-disclosed firm in year t (i.e.,all_hide=1)***
****If firm i in year t report n customers, out of which m customers are without specified names, the hide_ratio=m/n*************;

proc sort data=seg;by GVKEY CIK2 SRCDATE;run;
proc univariate data=seg noprint;
var reported sale_r SALECS;
by GVKEY CIK2 SRCDATE;
output out=c_report n=n_cus sum=report sale_rep sale_all;
run;

data c_report;
set c_report;
if report=0 then all_hide=1;
else all_hide=0;
hide_ratio=1-report/n_cus;
if sale_rep=. then sale_rep=0;
hide_ratio_sale=1-sale_rep/sale_all;
if hide_ratio=1 then hide_ratio_sale=1;
if cik2=. then delete;
run;


***Import the IDD data (replication from Klasa et al. (2014)********;
data idd;
set idd.idd;
run;

data comp;
set idd.real_loca;
keep stinc f_fdate ba_state CIK;
if ba_state="" then delete;
run; 

proc sql;
create table post as
select comp.*,idd.* from
comp as a left join idd as b
on a.ba_state=b.State ;
quit;

data post;
set post;
year=int(f_fdate/10000);
mon=int(f_fdate/100)-int(f_fdate/10000)*100;
day=f_fdate-year*10000-mon*100;
datadate=mdy(mon, day, year);
format datadate DATE9.;
dis_b=INTCK('MONTH',DATADATE,begdate) ;
dis_e=INTCK('MONTH',DATADATE,enddate);
if dis_e>1000 then dis_e=.;
if begdate<datadate<enddate then post=1;else post=0;
run;
	
data post;
set post;
if post=1 then dis_adopt=abs(dis_b);
if post=0 then dis_adopt=-1*abs(dis_b);
run;
	
data post;
set post;
if post=1 then dis_against=-1*abs(dis_e);
if post=0 then dis_against=abs(dis_e);
run;

data post;
set post;
if ba_state="" then delete;
run;

data post;
set post;
if mon>6 then fyear=year;
else fyear=year-1;
run;

proc sort nodupkey data=post;
by cik  fyear;
run;

data c_report;
set c_report;
year=year(SRCDATE);
month=month(srcdate);
if 1<=month<=5 then fyear=year-1;
if 6<=month<=12 then fyear=year;
run;

proc sql;
create table c_report2 as
select c_report.*,post.* from
post as a left join c_report as b
on a.cik=b.cik2 and a.year=b.year ;
quit;

proc sort nodupkey data=c_report2 ;
by gvkey fyear;
run;***39961 obs******************; 



/*****************************************************************************************
Calculating main control variables in Table 2 following Ellis et al. [2012]
*******************************************************************************************/

libname pead "C:\Users\Dropbox\IDD and disclosure\raw data\control variables";

/*Import accounting, customer-segment dataset, and inflation rate data*/
data north;
set pead.north;
run;/*North America accounting data*/

data customer;
set idd.segment;
run;/*customer segment data*/

data inflation;
set pead.inflation_USA;
run;/*Inflation, GDP deflator (annual%); data source:WorldBank*/


/*data cleaning*/
data north1;
set north;
sale=abs(sale);
if xrd=. then xrd_new=0;
else xrd_new=xrd;
if xrd=. then mis_rd=1;
else mis_rd=0;
if xad=. then xad_new=0;
else xad_new=xad;
if intan=. then intan_new=0;
else intan_new=intan;
if gdwl=. then gdwl_new=0;
else gdwl_new=gdwl;
run;

data customer1;
set customer;
year=year(srcdate);
month=month(srcdate);
if 1<=month<=5 then fyear=year-1;
if 6<=month<=12 then fyear=year;
run;


/*Merge: get supplier-customer-year level observations*/
proc sort data=north1;
by  fyear;
run;


proc sort data=inflation;
by fyear;
run;

data proprietary1;
merge inflation north1;
by fyear;
run;


data  proprietary1;
set  proprietary1;
if fic ne "USA" then delete;
run;

proc sort nodupkey data=proprietary1;
by gvkey fyear;
run;


proc sort data=proprietary1; by gvkey fyear;run;
proc sort data=Customer1; by gvkey fyear;run;

data proprietary2;merge proprietary1  Customer1; by gvkey fyear;run;

data proprietary3;
set proprietary2;
sale_ratio=salecs/sale;
if sale_ratio>=0.1 then salecs_dum=1;
else salecs_dum=0;
run;

data proprietary3;
set proprietary3;
keep gvkey fyear salecs_dum;run;

proc sort data=proprietary3; by  gvkey fyear salecs_dum;run;
data proprietary3;set proprietary3;  by  gvkey fyear salecs_dum;if last.fyear;run;

proc sort data=proprietary3;  by  gvkey fyear;run;
proc sort data=proprietary1;  by  gvkey fyear;run;
data proprietary4;merge proprietary1 proprietary3; by  gvkey fyear;run;

proc sort nodupkey data=proprietary4;
by gvkey fyear;
run;


/*Calculate controls: R&D to Sales,Net intangibles to assets,Advertising to sales,Ln(assets)*/
data proprietary4;
set proprietary4;
RD_to_Sales=xrd_new/abs(sale);
Intangible_to_AT=(intan_new-gdwl_new)/at;
Advertise_to_sale=xad_new/abs(sale);
LnAT=log(AT);
run;


/*Calculate supplier industry HHI--four digit sic code*/

proc sort nodupkey data=proprietary4;
by gvkey fyear;
run;
proc sort data=proprietary4;
by sic fyear;
run;

proc sql;  
create table proprietary5 as  
select *,sum(sale) as sum_sale from proprietary4
group by sic,fyear; 
quit;
 
data proprietary5;
set proprietary5;
ratio_square=(sale/sum_sale)*(sale/sum_sale);
run;

proc sort data=proprietary5;
by sic fyear;
run;

proc sql;  
create table proprietary6 as  
select *,sum(ratio_square) as HHI from proprietary5
group by sic,fyear; 
quit;

proc univariate data=proprietary6; var hhi;run;

/*Calculate Big N auditors dummy*/
data proprietary6;
set proprietary6;
if au="1" or au="2" or au="3" or au="4" or au="5" or au="6" or au="7" or au="8"
then bigN=1;
else bigN=0;
run;

/*Note:we classify auditors as Big N and non-Big N firms (Compustat item 149).
Because of mergers and collapses during our sample period, the data begin with eight
large auditors, which is reduced to four at the end of our sample period(1976-2013).
More details please see: Does Auditor Quality and Tenure Matter to Investors-JAR(2004)*/

/*Calculate supplier industry abnormal profit persistence*/
data proprietary13;
set proprietary6;
ROA=oibdp/at;
run;

proc sort nodupkey data=proprietary13;
by gvkey fyear;
run;

proc sort data=proprietary13;
by sic fyear;
run;

proc univariate data=proprietary13 noprint;
var ROA;
output out=a0 
mean=mea_ROA
median=med_ROA;
by sic fyear;
run;

proc sort data=a0;by sic fyear;run;
proc sort data=proprietary13;by sic fyear;run;

data proprietary14;
merge proprietary13 a0;
by sic fyear;
   adjmean_ROA=ROA-mea_ROA;
   adjmedian_ROA=ROA-med_ROA;
run;
/*lag 1:adjmean_ROA,adjmedian_ROA*/
data proprietary15; 
set proprietary14; 
keep gvkey fyear adjmean_ROA adjmedian_ROA; 
run;
       
data proprietary16; 
set proprietary15; 
rename adjmean_ROA=lag1_adjmean_ROA adjmedian_ROA=lag1_adjmedian_ROA;
fyear=fyear+1; 
run;       

proc sort data=proprietary14;
by gvkey fyear; 
run;

proc sort data=proprietary16; 
by gvkey fyear;
run;

data proprietary17; 
merge proprietary14 proprietary16; 
by gvkey fyear; 
run;

/*create dummy*/
data proprietary18; 
set proprietary17; 
if lag1_adjmean_ROA<=0 then DL=1;
else DL=0;
if lag1_adjmean_ROA>0 then DP=1;
else DP=0;
run;

data proprietary19; 
set proprietary18;
rename adjmean_ROA=ABROA lag1_adjmean_ROA=lag1_ABROA;
run;

data proprietary20; 
set proprietary19;
interact_DL=lag1_ABROA*DL;
interact_DP=lag1_ABROA*DP;
if ABROA=. or  interact_DL=. or interact_DP=. then delete;
run;

proc sort data=proprietary20;
by sic;
run;

proc reg data=proprietary20
outest=proprietary21(keep=sic intercept interact_DL interact_DP)noprint;
model ABROA=interact_DL interact_DP;
by sic;
run;

data proprietary22; 
set proprietary21;
rename interact_DP=ABROA_persistence;
run;


/*merge:proprietary12 and  proprietary22*/
proc sort data=proprietary6;
by sic; 
run;

proc sort data=proprietary22; 
by sic;
run;

data proprietary23; 
merge proprietary6 proprietary22; 
by sic; 
run;

proc sort nodupkey data=proprietary23;
by gvkey fyear;
run;


***Merge with the headquarter data***********;

data control;set proprietary23;run;

proc sort data=c_report2;by gvkey fyear;run;
proc sort data=control;by gvkey fyear;run;

**The controls have to be ex ante measures****;

proc sql;
create table c_report3 as
select c_report2.*,control.* from
c_report2 as a left join control as b
on a.gvkey=b.gvkey and a.fyear=b.fyear+1 ;
quit;



***Import the fiscal quarter price and share outstanding ***********;

data mkv;
set "C:\Users\Dropbox\IDD and disclosure\raw data\control variables\fiscal_quarter_price";
run;

data mkv;
set mkv;
mkv=CSHOQ*PRCCQ;
run;

proc sort data=mkv;
by gvkey FYEARQ FQTR;
run;

data mkv;
set mkv;
if FQTR=2 then mkv_2=mkv;
run;

data mkv2;
set mkv;
if mkv_2=. then delete;
run;

data mkv_3;
set mkv;
if mkv_2^=. then delete;run;

data miss;
set mkv_3;
if FQTR^=2 then delete;
miss=1;
run;

data miss;
set miss;
keep gvkey FYEARQ miss;run;

proc sort data=miss;by gvkey FYEARQ;run;
proc sort data=mkv_3; by gvkey FYEARQ;run;

data mkv_4;merge miss mkv_3;by gvkey FYEARQ;run;

data mkv_4;
set mkv_4;
if miss^=1 then delete;run;
data mkv_4;
set mkv_4;
if mkv=. then delete;
run;

proc sort data=mkv_4;by gvkey FYEARQ FQTR;run;

data mkv_5;
set mkv_4;by gvkey FYEARQ FQTR; if first.FYEARQ;run;
data mkv_6;
set mkv_5;
FQTR=2;
mkv2=mkv;
run;

data mkv7;
set mkv_6 mkv2;
run;

proc sort nodupkey data= mkv7;
by gvkey fyearq;
run;

data mkv7;
set mkv7;
keep GVKEY fyearq mkv;
run;

****Merge back************;
proc sql;
create table c_report4 as
select c_report3.*,mkv7.* from
c_report3 as a left join mkv7 as b
on a.gvkey=b.gvkey and a.fyear=b.fyearq ;
quit;
