/****************************************************************************************************
********* This program (filename CODE2) is used to construct additional variables. (1) Seo:*********
********* the SEO indicator obtained from SDC;(2) ma: the M&A indicator obtained from SDC; *********
********* (3) State_unemployment_rate: obtain from U.S. Bureau of Labor Statistics; (4)    *********
********* GDP_growth_rate: obtain from Bureau of Economic Analysis ; (5): share: the share *********
********* outstanding from Compustat; (6)INS: 13 F data (institutional investor); (7) po:  *********
********* hand cleaned election data;(8) FLU: product market competition (fluidity);(9)ind:*********
********* product market competition (level) ((8)&(9) are provided by Gordon Phillips) ;   *********
********* (10)reg1:the c_report4 that constructed by CODE1.                                *********                                                                                          
****************************************************************************************************
****************************************************************************************************/

libname con "C:\Users\Dropbox\IDD and disclosure\raw data";
data seo;
set con.sdcseo;
run;
data ma;
set con.sdcma;
run;

PROC IMPORT OUT= WORK.gdp
            DATAFILE= "C:\Users\Dropbox\IDD and disclosure\raw data\GDP_growth_rate.csv" 
            DBMS=CSV REPLACE;
     GETNAMES=YES;
     DATAROW=2; 
RUN;
PROC IMPORT OUT= WORK.UNEM
            DATAFILE= "C:\Users\Dropbox\IDD and disclosure\raw data\State_unemployment_rate.csv" 
            DBMS=CSV REPLACE;
     GETNAMES=YES;
     DATAROW=2; 
RUN;

proc sort data=unem; by state year;run;
proc univariate data=unem noprint;var Value; by state year; output out=unem2 mean=unemployment;
run;


data ma;
set ma;
rename ma=manextyear;
label ma=manextyear;
FYEAR=fyear-1;
run;

data seo;
set seo;
rename seo=seonextyear;
label seo=seonextyear;
FYEAR=fyear-1;
run;

*******The reg1 is the same as the c_report4 that constructed by CODE1**************************;

PROC IMPORT OUT= WORK.reg1
            DATAFILE= "C:\Users\Dropbox\IDD and disclosure\regression data\all data.dta" 
            DBMS=STATA REPLACE;
RUN;


/*************************************************************************************************
Other controls used in the supplementary tests 

*************************************************************************************************/

proc sql;
create table reg2 as
select reg1.*, ma.manextyear from
reg1 as a left join ma as b
on a.gvkey=b.gvkey and a.fyear=b.fyear ;
quit;
	
proc sql;
create table reg3 as
select reg2.*, seo.seonextyear from
reg2 as a left join seo as b
on a.gvkey=b.gvkey and a.fyear=b.fyear ;
quit;


proc sql;
create table reg4 as
select reg3.*, gdp.GDPGR from
reg3 as a left join gdp as b
on a.ba_state=b.State and a.fyear=b.year+1 ;
quit;

proc sql;
create table reg5 as
select reg4.*, unem2.unemployment from
reg4 as a left join unem2 as b
on a.ba_state=b.State and a.fyear=b.year+1 ;
quit;


***IMPORT THE PRODUCT MARKET COMPETITION MEASURES BASED ON TEXT ANALYSIS************************;
PROC IMPORT OUT= WORK.FLU 
            DATAFILE= "C:\Users\Dropbox\IDD and disclosure\raw 
data\product market competition\FICHHIdata_extend.txt" 
            DBMS=TAB REPLACE;
     GETNAMES=YES;
     DATAROW=2; 
RUN;PROC IMPORT OUT= WORK.FLU3 
            DATAFILE= "C:\Users\Dropbox\IDD and disclosure\raw 
data\product market competition\TNIC3HHIdata_extend.txt" 
            DBMS=TAB REPLACE;
     GETNAMES=YES;
     DATAROW=2; 
RUN;

PROC IMPORT OUT= WORK.ind 
            DATAFILE= "C:\Users\Dropbox\IDD and disclosure\raw 
data\product market competition\fic_data_extended.txt" 
            DBMS=TAB REPLACE;
     GETNAMES=YES;
     DATAROW=2; 
RUN;

data reg5;
set reg5;
newgvkey=gvkey*1;run;

proc sql;
create table reg6 as
select reg5.*, ind.icode300 from
reg5 as a left join ind as b
on a.newgvkey=b.gvkey and a.fyear=b.year+1 ;
quit;

proc sql;
create table reg7 as
select reg6.*, flu.fic300hhi from
reg6 as a left join flu as b
on a.icode300=b.icode300 and a.fyear=b.year+1 ;
quit;

proc sql;
create table reg8 as
select reg7.*, flu3.tnic3hhi, flu3.tnic3tsimm from
reg7 as a left join flu3 as b
on a.newgvkey=b.gvkey and a.fyear=b.year+1 ;
quit;

*****analysts coverage**********;
	%let wrds = wrds.wharton.upenn.edu 4016;
 options comamid=TCP remote=WRDS;
 signon username=_prompt_;

 ************ analyst coverage in fiscal year t per IBES detailed forecasts; 
 rsubmit;


 libname home '.';

 %include '/wrds/ibes/samples/iclink.sas'; *creates IBES Ticker- CRSP permno Link*;
 endrsubmit;

 rsubmit;

 %let screen1 = where (indfmt='INDL' and datafmt='STD' and popsrc='D' and consol='C');

 data firmyrs; set COMP.funda; &screen1;
    fyenddt=datadate; fybegdt=intnx('monthly', fyenddt, -11);  
    format fyenddt fybegdt date9.;  
    keep FIC gvkey fybegdt fyenddt; 
 run;

 **** Get Permno;
 proc sql;
 create table firmyrs1 
 as select a.*, b.lpermno as permno
 from firmyrs as a left join crsp.ccmxpf_linktable as b
 on a.gvkey=b.gvkey and
 b.LINKTYPE in ("LU","LC","LD","LF","LN","LO","LS","LX")and
 b.usedflag=1 and
 (b.LINKDT <= a.fyenddt or missing(b.LINKDT) = 1) 
 and (a.fyenddt <= b.LINKENDDT or b.LINKENDDT= .E)
 order by a.gvkey, a.fyenddt;
 quit;

 data input; set Firmyrs1; keep FIC gvkey permno fybegdt fyenddt; 
   where permno ne . and fyenddt ne .; 
 run;
 proc sort data=input nodupkey; by FIC gvkey permno fyenddt; run;

 proc sql; create table input1  
 as select a.*, b.ticker as IBEStick
 from input as a left join home.iclink (keep=ticker permno score where=(score in (0,1,2))) as b
 on a.permno=b.permno;
 quit;

 proc sql;
 create table Analysts_ann
 as select a.*, ANALYS    
 from input1 as a, IBES.detu_epsus as b
 where a.IBEStick=b.ticker and a.fybegdt <= ANNDATS <=a.fyenddt and usfirm=1;  
 quit;

 proc sort data=Analysts_ann nodupkey; by FIC gvkey permno IBEStick fyenddt ANALYS; run;

 proc univariate data=Analysts_ann noprint;
 var ANALYS;
 by FIC gvkey permno IBEStick fyenddt;
 output out=nAnalysts_ann n=nAnalysts;
 run;

 proc download data=nAnalysts_ann out=work.NumAnalysts_ann; run;

 endrsubmit;


data share;
set "C:\Research\data\comp\comp";
keep gvkey fyear cusip CSHO;
run;

DATA INS;
SET "C:\Users\Dropbox\IDD and disclosure\raw data\control variables\io";
RUN;

data ins;
set ins;
ratio=shares/(SHROUT1*1000000);
run;

data ins;
set ins;
if ratio>0.05 then io=1; else io=0;
run;

proc sort data=ins;
by cusip FDATE;run;
proc univariate data=ins noprint;
var SHARES ratio io;by cusip FDATE; output out=Io sum=totalIo ratio2 io;run;
data io;
set io;
year=year(FDATE);
run;

proc sort data=io;
by cusip year;run;

proc univariate data=Io noprint;
var totalIo ratio2 io;
by cusip year;
output out=io2 mean=totalio ratio2 io;
run;

data io2;
set io2;
if cusip="" then delete;run;

data share;
set share;
cusip8=substr(cusip,1,8);
run;
	
proc sql;
create table IO3 as
select IO2.*, share.* from
io2 as a left join share as b
on a.cusip=b.cusip8 and a.year=b.fyear ;
quit;

DATA IO3;
SET IO3;
RATIO3=totalio/(CSHO*1000000);
RUN;

DATA IO3;
SET IO3;
KEEP CUSIP GVKEY RATIO2 RATIO3 FYEAR io;
RUN;

 ******
 DATA analyst;
 set "C:\Users\Dropbox\IDD and disclosure\raw data\control variables\analysts";
 run;
 

proc sql;
create table reg9 as
select reg8.*, Io3.ratio2, io3.ratio3, io3.io from
reg8 as a left join io3 as b
on a.gvkey=b.gvkey and a.fyear=b.fyear+1 ;
quit;

data reg9;
set reg9;
if gvkey="" then delete;
run;

proc sort data=reg9 nodupkey; by gvkey fyear; run;

data analyst;set analyst; fyear=year(fyenddt);run;
proc sort data=analyst; by gvkey fyenddt;run;


proc sql;
create table reg10 as
select reg9.*, analyst.nAnalysts from
reg9 as a left join analyst as b
on a.gvkey=b.gvkey and a.fyear=b.fyear+1 ;
quit;

proc sort data=reg10 nodupkey; by gvkey fyear; run;




PROC IMPORT OUT= WORK.po 
            DATAFILE= "C:\Users\Dropbox\IDD and disclosure\raw 
data\political data\political.xls" 
            DBMS=EXCEL REPLACE;
     RANGE="'93-2011$'"; 
     GETNAMES=YES;
     MIXED=NO;
     SCANTEXT=YES;
     USEDATE=YES;
     SCANTIME=YES;
RUN;

data po;
set po;
if ind^="" then ind2=ind*1; else ind2=0;
run;


proc sql;
create table reg11 as
select reg10.*, po.re, po.de, po.ind2 from
reg10 as a left join po as b
on a.ba_state=b.state and a.fyear=b.year ;
quit;



****comp is the accounting data from compustat**********;
data comp;
set "C:\Research\data\comp\comp";
run;

data comp;
set comp;
cashflow=oancf/at;
MB=( prcc_f*csho+at-ceq-txdb)/at;
leverage= (dlc+ dltt)/(dltt+dlc+prcc_f*csho);
ROA=ebitda/at;
cash=che/at;
tang=ppent/at;
size=log(at);
if xrd=. then rd=0;else rd=xrd/at;
if xrd=. then mis_rd=1;else mis_rd=0;
run;

data issue;
set issue;
if EQUITYFIN>0 then equity=1;else equity=0;
if DEBTFIN>0 then debt=1; else debt=0;
if EQUITYFIN>0 then issue=1;
if DEBTFIN>0 then issue=1;
run;

data issue;
set issue;
if issue=. then issue=0;
run;

proc sql;
create table issue2 as
select issue.*,comp.SICH, comp.rd,comp.mis_rd,comp.cashflow,comp.size,comp.tang,comp.cash,comp.mb,comp.leverage,comp.roa  from
issue as a left join comp as b
on a.gvkey=b.gvkey and a.fyear=b.fyear-1 ;
quit;


proc sort nodupkey data=issue2;
by gvkey   fyear;
run;

proc logistic data =issue2;
CLASS sich fyear;
model equity =mb roa leverage cash size tang rd mis_rd sich fyear
/link=logit rsquare;
output out = psdataset1 pred = ps_equity
xbeta=logit_ps;
run;

proc logistic data = psdataset1;
CLASS sich fyear;
model debt =mb roa leverage cash size tang rd mis_rd sich fyear
/link=logit rsquare;
output out = psdataset2 pred = ps_debt
xbeta=logit_ps;
run;


proc logistic data = psdataset2;
CLASS sich fyear;
model issue =mb roa leverage cash size tang rd mis_rd sich fyear
/link=logit rsquare;
output out = psdataset3 pred = ps_all
xbeta=logit_ps;
run;


proc sql;
create table reg12 as
select reg11.*, psdataset3.ps_equity, psdataset3.ps_debt,psdataset3.ps_all,psdataset3.DEBTFIN, psdataset3.EQUITYFIN from
reg11 as a left join psdataset3 as b
on a.gvkey=b.gvkey and a.fyear=b.fyear ;
quit;

proc sort nodupkey data=reg12;
by gvkey fyear;
run;

**********Construct industry age and stability*********************;
data comp_2;
set "C:\Users\Dropbox\comp";
run;
data c1;

set comp;
keep datadate fyear sic cusip gvkey sale;
run;

data age;
set "C:\Users\Dropbox\IDD and disclosure\raw data\age\q21e3b6b2f0bbe3e5";
run;

data c1;
set c1;
cusip8=substr(cusip,1,8);
run;

data age;
set age;
cusip8=cusip;
run;

proc sort data=c1; by cusip8;run;
proc sort data=age; by cusip8;run;

data c2;merge c1 age; by cusip8;run;

data c2;
set c2;
if gvkey="" then delete;run;

data c2;
set c2;
his=datadate-BEGDAT;
run;

data c2;set c2;
if his<0 then his2=0;else his2=his;
if his=. then his2=.;
run;
data c2;
set c2;
if sale="" then delete;
run;

data c2;
set c2;
sic2=substr(sic,1,2)*1;
run;
proc sort data=c2;by sic2 fyear;run;

data c3;
set c2;
if fyear>2010 then delete;
if fyear<1994 then delete;
run;

proc sort data=c3;by sic2;run;

proc univariate data=c3 noprint;
var his2 ;
by sic2; 
output out=sum mean= mean_age std= std_age max= max_age;
run;

proc sort data=c2;by sic2 fyear;run;

proc univariate data=c2 noprint;
var SALE;
by sic2 fyear;
output out=sum2 mean=mean_sale median=median_sale;
run;

data sum2;
set sum2;
start=fyear-9;run;

data base;
set sum2;
keep sic2 fyear start;
run;

proc sql;
create table std as
select base.*,c2.sale from
base as a left join c2 as b
on a.sic2=b.sic2 and a.start<=b.fyear<=a.fyear;
quit;

proc sort data=std;by sic2 fyear;run;

proc univariate data=std noprint;
var sale; by sic2 fyear;output out=sale_std std=sale_std; run;

